@@ -293,7 +293,7 @@ def multiline_strings_are_exactly_equal(val, ref):
     """
     A checker to be used with harnesses which generate multi-line strings
     where the outputs can reasonable be expected to be exactly equal.
-    Displays results more nicely than omni_compare because it knows
+    Displays results more nicely than `omni_compare` because it knows
     they'll be strings.
     """
     if not isinstance(val, str) or not isinstance(ref, str):
@@ -1065,6 +1065,18 @@ _STUB_CHARS = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
 _STUB_LEN = len(_STUB_CHARS)
 
 
+def reset_stubs():
+    """
+    Resets the stub counter. Useful to ensure that equivalent data
+    structures are equivalent when made hashable via equivalent stubs,
+    although order really matters (i.e., recursive dictionaries with
+    different key orders will not necessarily end up with the same
+    stubs).
+    """
+    global _STUB_N
+    _STUB_N = 0
+
+
 def next_stub():
     """
     Generates a unique string to indicate the presence of a recursive
@@ -1083,7 +1095,7 @@ def next_stub():
 
 def make_hashable(dataStructure, memo=None):
     """
-    Turns a data structure that may contain lists, tuples, dictionaries,
+    Takes a data structure that may contain lists, tuples, dictionaries,
     and/or sets as well as strings, numbers, booleans, and Nones, and
     turns it into a similar structure using tuples in place of lists,
     sets, and dictionaries that can be hashed. Uses a ID-based memo to
@@ -1091,6 +1103,7 @@ def make_hashable(dataStructure, memo=None):
     of already-visited objects.
     """
     if memo is None:
+        reset_stubs()
         memo = {}
 
     # If this is a re-visit, return a stub string
@@ -1101,14 +1114,14 @@ def make_hashable(dataStructure, memo=None):
     memo[id(dataStructure)] = next_stub()
 
     if isinstance(dataStructure, (list, tuple)):
-        return tuple([make_hashable(x) for x in dataStructure])
+        return tuple([make_hashable(x, memo) for x in dataStructure])
     elif isinstance(dataStructure, set):
         # We sort sets so that ordering is irrelevant
-        return tuple([make_hashable(x) for x in sorted(dataStructure)])
+        return tuple([make_hashable(x, memo) for x in sorted(dataStructure)])
     elif isinstance(dataStructure, dict):
         # We sort keys so that ordering is irrelevant
         return tuple(
-            (key, make_hashable(dataStructure[key]))
+            (key, make_hashable(dataStructure[key], memo))
             for key in sorted(dataStructure)
         )
     else:
@@ -1758,12 +1771,13 @@ def make_structure_comparator(
                 moved_missing = []
                 moved_extra = []
                 for midx, miss in enumerate(missing):
-                    if miss in extra:
-                        eidx = extra.index(miss)
-                        # TODO: What if multiple copies are out-of-order?
-                        moved_missing.append(midx)
-                        moved_extra.append(eidx)
-                        moved.append(miss)
+                    for eidx, xtra in enumerate(extra):
+                        if item_matcher(miss, xtra):
+                            # TODO: What if multiple copies are out-of-order?
+                            moved_missing.append(midx)
+                            moved_extra.append(eidx)
+                            moved.append(miss)
+                            break
 
                 new_missing = [
                     missing[idx]
@@ -2048,17 +2062,32 @@ treat a string as long.
 """
 
 
-def omni_compare(val, ref):
+def omni_compare(val, ref, memo=None):
     """
     A one-size-kinda-fits-all comparison method (although note that it
     assumes order matters in lists and tuples).
 
     Tries its best to give a concise-ish description of differences when
     they exist.
-
-    TODO: Recursive data structures
     """
-    if val == ref:
+    if memo is None:
+        memo = {}
+
+    vid = id(val)
+    rid = id(ref)
+    if vid in memo and rid in memo[vid]:
+        return True
+        # Recursive comparison that's already in-progress
+
+    memo.setdefault(vid, set()).add(rid)
+
+    # TODO: Better/safer here!
+    try:
+        matched = val == ref
+    except RecursionError:
+        matched = False # might be, but we'll have to check further
+
+    if matched:
         return {
             "status": "accomplished",
             "explanation": (
@@ -2144,12 +2173,19 @@ def omni_compare(val, ref):
                     }
 
         elif isinstance(val, (list, tuple)): # same-type sequences
+            # TODO: Less inefficient here!
             # TODO: More detailed explication of deep differences here?
-            result = OMNI_ORDERED_COMP(val, ref)
-            return result
+            return make_structure_comparator(
+                order_matters=True,
+                item_matcher=lambda val, ref: omni_compare(val, ref, memo)
+            )(val, ref)
         elif isinstance(val, (set, dict)): # both sets or dicts
+            # TODO: Less inefficient here!
             # TODO: More detailed explication of deep differences here?
-            return OMNI_UNORDERED_COMP(val, ref)
+            return make_structure_comparator(
+                order_matters=True,
+                item_matcher=lambda val, ref: omni_compare(val, ref, memo)
+            )(val, ref)
         else: # not sure what kind of thing this is...
             return {
                 "status": "failed",
@@ -2160,22 +2196,3 @@ def omni_compare(val, ref):
                     f" the same."
                 )
             }
-
-
-OMNI_ORDERED_COMP = make_structure_comparator(
-    order_matters=True,
-    item_matcher=omni_compare
-)
-"""
-Structure comparator that cares about ordering which uses omni_compare
-to match items.
-"""
-
-OMNI_UNORDERED_COMP = make_structure_comparator(
-    order_matters=False,
-    item_matcher=omni_compare
-)
-"""
-Structure comparator that doesn't care about ordering which uses
-omni_compare to match items.
-"""
