@@ -1937,6 +1937,7 @@ def build_context_value_displayer(
                         rep = html_tools.big_repr(context[key])
                     except TypeError:
                         rep = repr(context[key])
+                rep = html_tools.escape(rep)
                 return f"<pre class='context-value'>{rep}</pre>"
             else:
                 # failed to create the context key we're looking for!
@@ -1953,6 +1954,7 @@ def build_context_value_displayer(
                         rep = html_tools.big_repr(context[key])
                     except TypeError:
                         rep = repr(context[key])
+                rep = html_tools.escape(rep)
                 rep_html = f"<pre class='context-value'>{rep}</pre>"
             else:
                 rep = ""
@@ -1969,6 +1971,7 @@ def build_context_value_displayer(
                         ref_rep = html_tools.big_repr(context["ref_" + key])
                     except TypeError:
                         ref_rep = repr(context["ref_" + key])
+                ref_rep = html_tools.escape(ref_rep)
                 ref_rep_html = f"<pre class='context-value'>{ref_rep}</pre>"
             else:
                 ref_rep = ""
