@@ -406,13 +406,13 @@ def payload_description(
     elif base_constructor == harness.create_read_variable_payload:
         varname = constructor_args["varname"]
         if obfuscated:
-            topic = "The value of <code>{varname}</code>"
+            topic = f"The value of <code>{varname}</code>"
             details = (
                 f"We will inspect the value of"
                 f" <code>{varname}</code>."
             )
         else:
-            topic = "The value of <code>{varname}</code>"
+            topic = f"The value of <code>{varname}</code>"
             details = (
                 f"We inspected the value of <code>{varname}</code>."
             )
