@@ -241,7 +241,12 @@ def create_run_function_payload(
     made before they are passed to the target function (note that keyword
     argument keys are not copied, although they should be strings in any
     case). The "args" and "kwargs" slots will also get copies of the
-    arguments, not the original values.
+    arguments, not the original values, and these will be separate copies
+    from those given to the function, so they'll retain the values used
+    as input even after the function is finished. However, "used_args"
+    and "used_kwargs" slots will be added if `copy_args` is set to true
+    that hold the actual arguments sent to the function so that any
+    changes made by the function can be measured if necessary.
 
     If the function name or any of the argument values (or keyword
     argument keys) are `potluck.context_utils.ContextualValue` instances,
@@ -326,13 +331,19 @@ def create_run_function_payload(
                 real_kwargs[key] = value
                 initial_kwargs[key] = value
 
-        return {
+        result = {
             "value": fn(*real_posargs, **real_kwargs),
             "function": fname,
             "args": initial_posargs,
-            "kwargs": initial_kwargs
+            "kwargs": initial_kwargs,
         }
 
+        if copy_args:
+            result["used_args"] = real_posargs
+            result["used_kwargs"] = real_kwargs
+
+        return result
+
     return payload
 
 
@@ -713,9 +724,9 @@ def run_in_sandbox(payload):
 def with_setup(payload, setup):
     """
     Creates a modified payload which runs the given setup function
-    (with the incoming context as an argument) right before running the
-    base payload. The setup function's return value is used as the
-    context for the base payload.
+    (with the incoming context dictionary as an argument) right before
+    running the base payload. The setup function's return value is used
+    as the context for the base payload.
 
     Note that based on the augmentation order, function calls made during
     the setup WILL NOT be captured as part of a trace if
@@ -737,9 +748,9 @@ def with_setup(payload, setup):
 def with_cleanup(payload, cleanup):
     """
     Creates a modified payload which runs the given cleanup function
-    (with the original payload's result as an argument) right after
-    running the base payload. The return value is the cleanup function's
-    return value.
+    (with the original payload's result, which is a context dictionary,
+    as an argument) right after running the base payload. The return
+    value is the cleanup function's return value.
 
     Note that based on the augmentation order, function calls made during
     the setup WILL NOT be captured as part of a trace if
