@@ -182,7 +182,7 @@ def load_instructions_html(spec):
             # TODO: Handle encoding guess!
             with open(target, 'rb') as fin:
                 src_bytes = fin.read()
-                data = base64.b64encode(src_bytes).decode("utf-8")
+                data = base64.standard_b64encode(src_bytes).decode("utf-8")
             # Build a data URI and update the src attribute:
             data_uri = "data:{};base64,{}".format(mime, data)
             tag['src'] = data_uri
@@ -201,22 +201,11 @@ def load_instructions_html(spec):
 # Module loading & parsing #
 #--------------------------#
 
-def link_mapping(spec):
-    """
-    Extracts the standard symlink mapping for sandbox files from the
-    given specification, based on its helper_files and starter_path
-    properties. The result is suitable as a sandbox_links argument to
-    `create_module_in_sandbox`.
-    """
-    return {
-        os.path.abspath(os.path.join(spec.starter_path, helper)): helper
-        for helper in spec.helper_files
-    }
-
 
 def create_module_in_sandbox(
     node,
     filename,
+    sandbox_dir=None,
     sandbox_links=None,
     sandbox_files=None,
     on_disk=None
@@ -224,10 +213,18 @@ def create_module_in_sandbox(
     """
     Given an AST node and a filename, creates a temporary sandbox
     directory, runs the code in the sandbox to create a module object,
-    and returns the module object that was created. If extra files are
-    needed in the sandbox, a dictionary mapping absolute paths to
-    paths-in-sandbox can be supplied and those files will be symlinked
-    in (see `link_mapping`). Alternatively, an equivalently-structured
+    and returns the module object that was created.
+
+    An explicit sandbox directory that's already set up may be provided
+    via the `sandbox_dir` parameter. If none is provided, a new sandbox
+    will be created and then destroyed in the course of running this
+    function. If an existing sandbox will be used, `sandbox_links` and
+    `sandbox_files` are ignored.
+
+    If a pre-existing sandbox isn't provided and extra files are needed
+    in the sandbox, a dictionary mapping absolute paths to
+    paths-in-sandbox can be supplied and those files will be symlinked in
+    (see `link_mapping`). Alternatively, an equivalently-structured
     sandbox_files directory may be supplied to copying files rather than
     creating links, which is typically less efficient, but desirable if
     those files will be modified.
@@ -236,29 +233,42 @@ def create_module_in_sandbox(
     code was parsed from, and will be used to provide a __file__
     variable while the code runs.
     """
-    with tempfile.TemporaryDirectory(
-        suffix="__tmp",
-        dir=SANDBOX_DIR
-    ) as tmpdir:
-        # Create symlinks
-        if sandbox_links is not None:
-            for filepath in sandbox_links:
-                to = os.path.join(tmpdir, sandbox_links[filepath])
-                os.symlink(filepath, to)
-
-        # Copy files
-        if sandbox_files is not None:
-            for filepath in sandbox_files:
-                to = os.path.join(tmpdir, sandbox_files[filepath])
-                shutil.copy(filepath, to)
+    if sandbox_dir is not None:
+        # A sandbox directory has been provided ready-to-use; ignore
+        # sandbox_links and sandbox_files.
 
         # Create the module
         result = create_module_from_code(
             node,
             filename,
             on_disk=on_disk,
-            sandbox=tmpdir
+            sandbox=sandbox_dir
         )
+    else:
+        # We need to create our own sandbox directory
+        with tempfile.TemporaryDirectory(
+            suffix="__tmp",
+            dir=SANDBOX_DIR
+        ) as tmpdir:
+            # Create symlinks
+            if sandbox_links is not None:
+                for filepath in sandbox_links:
+                    to = os.path.join(tmpdir, sandbox_links[filepath])
+                    os.symlink(filepath, to)
+
+            # Copy files
+            if sandbox_files is not None:
+                for filepath in sandbox_files:
+                    to = os.path.join(tmpdir, sandbox_files[filepath])
+                    shutil.copy(filepath, to)
+
+            # Create the module
+            result = create_module_from_code(
+                node,
+                filename,
+                on_disk=on_disk,
+                sandbox=tmpdir
+            )
 
     return result
 
