@@ -61,6 +61,30 @@ def all_row_trails(report_or_row, trail_prefix=None):
             yield (report_or_row, below)
 
 
+class ExpectedWarning:
+    """
+    An expected warning provides a heads-up that a warning containing
+    certain text is expected, so that getting such a warning won't fail a
+    check.
+    """
+    def __init__(self, message_fragment=''):
+        """
+        A message fragment should be provided, or by default all warnings
+        will be ignored. If a fragment is provided, all warnings which
+        include that fragment as part of their raw HTML code string will
+        be ignored, but other warnings will not be.
+        """
+        self.fragment = message_fragment
+
+    def unexpected(self, warnings):
+        """
+        Returns all of the warnings from the given list which *aren't*
+        expected, given this expectation that certain warning(s) might be
+        present.
+        """
+        return [w for w in warnings if self.fragment not in w]
+
+
 class Expectation:
     """
     An expectation establishes that a specific goal should evaluate to a
@@ -290,11 +314,11 @@ def check_entire_report(
     require_default="accomplished"
 ):
     """
-    Given a report and a list of `Expectation` objects, this function
-    checks each of the expectations within the provided report,
-    returning a tuple containing True or False to indicate success or
-    failure, as well as a multi-line string explaining which checks
-    failed or that all checks succeeded.
+    Given a report and a list of `Expectation` and/or `ExpectedWarning`
+    objects, this function checks each of the expectations within the
+    provided report, returning a tuple containing True or False to
+    indicate success or failure, as well as a multi-line string
+    explaining which checks failed or that all checks succeeded.
 
     If require_default is provided, then all rubric rows in the report
     which don't have an explicit `Expectation` provided for them or a
@@ -305,23 +329,42 @@ def check_entire_report(
     explanation = "Some checks failed:\n"
     coverage = {}
     succeeded = True
+    unexpected_warnings = report["warnings"]
     for exp in all_expectations:
-        success, expl, path = exp.check(report)
-        if path is None:
-            if isinstance(exp.goal_spec, str):
-                gs = '#' + exp.goal_spec
-            else:
-                gs = ' → '.join(exp.goal_spec)
-            raise ValueError(
-                "Unable to find expected goal:\n{}\n{}".format(gs, expl)
-            )
-        c = coverage
-        for entry in path:
-            c = c.setdefault(entry, {})
-        c[None] = True
-        if not success:
-            explanation += expl + '\n'
-            succeeded = False
+        if isinstance(exp, ExpectedWarning):
+            # Filter out warnings
+            before = len(unexpected_warnings)
+            unexpected_warnings = exp.unexpected(unexpected_warnings)
+            if len(unexpected_warnings) == before: # nothing was filtered
+                succeeded = False
+                explanation += (
+                    f"  Expected at least one warning containing the text"
+                    f" '{exp.fragment}', but no such warning was present."
+                    f"\n  (or it was filtered by a different warning"
+                    f" expectation.)\n"
+                )
+
+        elif isinstance(exp, Expectation):
+            # Test goal status
+            success, expl, path = exp.check(report)
+            if path is None:
+                if isinstance(exp.goal_spec, str):
+                    gs = '#' + exp.goal_spec
+                else:
+                    gs = ' → '.join(exp.goal_spec)
+                raise ValueError(
+                    "Unable to find expected goal:\n{}\n{}".format(gs, expl)
+                )
+            c = coverage
+            for entry in path:
+                c = c.setdefault(entry, {})
+            c[None] = True
+            if not success:
+                explanation += expl + '\n'
+                succeeded = False
+
+        else:
+            raise TypeError(f"Invalid expectation type: {type(exp)}")
 
     default_count = 0
     if require_default is not None:
@@ -396,6 +439,19 @@ def check_entire_report(
             )
         )
 
+    # Check for warnings and replace/augment explanation
+    if len(unexpected_warnings) > 0:
+        wmsg = (
+            "The report included unexpected warnings:\n  "
+          + "\n ".join(unexpected_warnings)
+        )
+        if succeeded:
+            explanation = wmsg
+        else:
+            explanation = wmsg + '\n' + explanation
+
+        succeeded = False
+
     return (succeeded, explanation)
 
 
@@ -416,14 +472,11 @@ def example(username):
 
 def expect(status, *id_or_path):
     """
-    Creates an `Expectation` object and registers it under a given task
-    ID and username. Arguments are:
+    Creates an `Expectation` object and registers it under the current
+    example username. Arguments are:
 
     - status: The expected status. See `potluck.rubrics.Goal`.
-    - taskid: The task ID this expectation applies to.
-    - username: The username whose submission the expectation applies
-        to (usually a fake user set up for testing purposes).
-    - path: One or more additional strings specifying which goal we're
+    - id_or_path: One or more additional strings specifying which goal we're
         targeting (see `Expectation`). May also be a single string that
         starts with '#' to specify the goal using its identifier instead
         of a rubric-description-path. If it's a single string, it should
@@ -448,6 +501,24 @@ def expect(status, *id_or_path):
         )
 
 
+def expect_warnings(fragment=''):
+    """
+    Creates an `ExpectedWarning` object and registers it under the
+    current example username.
+
+    The `fragment` argument if omitted will cause all warnings to be
+    treated as expected, but if provided, only warnings whose raw HTML
+    message string contains that fragment as a substring will be treated
+    as expected.
+    """
+    mname = file_utils.get_spec_module_name()
+
+    EXPECTATIONS\
+        .setdefault(mname, {})\
+        .setdefault(CURRENT_EXAMPLE, [])\
+        .append(ExpectedWarning(fragment))
+
+
 def get_expectations(spec_module):
     """
     Returns all expectations for the given specification module, as a
