@@ -223,9 +223,9 @@ def wrapped_repr(obj, width=FMT_WIDTH, indentation=1, prefix='', memo=None):
             result += textwrap.indent(
                 wrapped_repr(item, width - indentation, indentation),
                 indent
-            ) + between
+            ) + ',' + between
 
-        return result[-len(between):] + '\n' + rdelim
+        return result[:-len(between)] + '\n' + rdelim
 
     elif type(obj) == dict: # need to wrap keys and values
         indent = ' ' * indentation
@@ -1156,6 +1156,84 @@ class Fakes(Snippet):
         return result
 
 
+class Files(Snippet):
+    """
+    A snippet which shows the contents of one or more files, without
+    running any code, in the same Jupyter-like format that file contents
+    are shown when `Snippet.show_file_contents` is used. Most
+    specialization methods don't work properly on this kind of snippet.
+    """
+    def __init__(self, sid, title, caption, filepaths, postscript=''):
+        """
+        A snippet ID, title, and caption are required, along with a list
+        of file paths. Each entry in the list should be a string path to
+        a starter file relative to the starter directory. The versions of
+        files in the solution directory will be used. But only files
+        present in both starter and solution directories will be
+        available.
+
+        A postscript is optional (see `Snippet`).
+        """
+        self.filepaths = filepaths
+
+        super().__init__(sid, title, caption, [], postscript)
+
+    def create_goal(self):
+        """
+        We override TestGroup.create_goal to just create a dummy goal
+        which depends on dummy contexts.
+        """
+        ctx_list = []
+        for filepath in self.filepaths:
+            with open(filepath, 'r', encoding="utf-8") as fileInput:
+                contents = fileInput.read()
+
+            def make_builder():
+                """
+                We need to capture 'filepath' on each iteration of the
+                loop, which is why this extra layer of indirection is
+                added.
+                """
+                nonlocal filepath, contents
+                return lambda _: {
+                    "path": filepath,
+                    "contents": contents
+                }
+
+            ctx_list.append(contexts.Context(builder=make_builder()))
+
+        return rubrics.NoteGoal(
+            self.taskid,
+            "fileSnippetGoal:" + self.base_name,
+            (
+                "NoteGoal for file snippet '{}'.".format(self.base_name),
+                "A file-displaying NoteGoal."
+            ),
+            test_in={ "contexts": ctx_list }
+        )
+
+    def render_context(self, context):
+        """
+        Renders a context with just file information as HTML markup for
+        displaying a file with %more magic. Includes a Jupyter-style
+        prompt for input as well as the contents of the file.
+        """
+        path = context["path"]
+        contents = context["contents"]
+
+        result = (
+            '<span class="prompt input">In []:</span>'
+            '<div class="snippet-input">'
+            '<code class="magic">%more {filename}</code>'
+            '</div>\n'
+            '<span class="prompt special">File</span>'
+            '<div class="snippet-filename">{filename}</div>\n'
+            '<div class="snippet-file-contents">{contents}</div>\n'
+        ).format(filename=path, contents=contents)
+
+        return result
+
+
 #---------#
 # Glamers #
 #---------#
